﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DataAccess.Models;
using System.Data.SqlClient;
using System.Data;

namespace DataAccess.Services
{
    public class Repository
    {
        private const string GET_NAMES = "GetNames"; 
        private string _connectionString;

        public const string STATUS_OK = "Ok"; 

        /// <summary>
        /// Repository
        /// </summary>
        /// <param name="connectionString"></param>
        public Repository(string connectionString)
        {
            _connectionString = connectionString; 
        }

        /// <summary>
        /// GetNames
        /// </summary>
        /// <param name="lastName"></param>
        /// <param name="firstName"></param>
        /// <param name="middleInitial"></param>
        /// <param name="ssn"></param>
        /// <param name="dob"></param>
        /// <returns></returns>
        public NameResults GetNames(string lastName = null, string firstName = null, string middleInitial = null, string ssn = null, DateTime? dob = null)
        {
            var names = new List<Name>(); 
            try
            {
                using (var connection = new SqlConnection(_connectionString))
                { 
                    connection.Open();
                    using (var command = new SqlCommand(GET_NAMES, connection))
                    {
                        command.CommandType = CommandType.StoredProcedure;
                        command.Parameters.Add(new SqlParameter("@LastName", SqlDbType.NVarChar, 50));
                        command.Parameters["@LastName"].Value = lastName;

                        command.Parameters.Add(new SqlParameter("@FirstName", SqlDbType.VarChar, 50));
                        command.Parameters["@FirstName"].Value = firstName;

                        command.Parameters.Add(new SqlParameter("@MiddleInitial", SqlDbType.Char));
                        command.Parameters["@MiddleInitial"].Value = middleInitial;

                        command.Parameters.Add(new SqlParameter("@SSN", SqlDbType.VarChar, 11));
                        command.Parameters["@SSN"].Value = ssn;

                        command.Parameters.Add(new SqlParameter("@DOB", SqlDbType.DateTime));
                        command.Parameters["@DOB"].Value = dob;

                        using (var reader = command.ExecuteReader())
                        {
                            while (reader.Read())
                            {
                                names.Add(
                                    new Name
                                    {
                                        Id = (int)reader["Id"],
                                        FirstName = reader["FirstName"].ToString(),
                                        LastName = reader["LastName"].ToString(),
                                        MiddleInitial = reader["MiddleInitial"].ToString(),
                                        SSN = reader["SSN"].ToString(),
                                        DOB = reader["DOB"].Equals(DBNull.Value) ? null : (DateTime?)reader["DOB"]
                                    });
                            }
                        }
                    }
                }

                return new NameResults
                {
                    Status = STATUS_OK,
                    Names = names
                };
            }
            catch (Exception ex)
            {
                return new NameResults
                {
                    Status = ex.Message,
                    Names = null
                };
            }
        }
    }
}
